#ifndef _SERIAL_H
#define _SERIAL_H

/* Includes */

#include "stm32f4xx_usart.h"
#include <stdint.h>
#include <string.h>

/* Defines */

#define SERIAL_UART		USART3
//#define SERIAL_FLOW_CONTROL	USART_HardwareFlowControl_RTS_CTS
#define SERIAL_FLOW_CONTROL	USART_HardwareFlowControl_None
#define SERIAL_UART_PORT	GPIOB
#define SERIAL_UART_TX_PIN	GPIO_Pin_10
#define SERIAL_UART_RX_PIN	GPIO_Pin_11
#define SERIAL_UART_TX_PIN_SRC	GPIO_PinSource10
#define SERIAL_UART_RX_PIN_SRC	GPIO_PinSource11
//#define SERIAL_UART_CTS_PIN
//#define SERIAL_UART_RTS_PIN
#define SERIAL_DMA_TX		DMA1_Stream3
#define SERIAL_DMA_RX		DMA1_Stream1
#define SERIAL_DMA_CHANNEL_TX	DMA_Channel_4
#define SERIAL_DMA_CHANNEL_RX	DMA_Channel_4

//#define SERIAL_BAUD_RATE	921600
#define SERIAL_BAUD_RATE	115200
//#define SERIAL_BAUD_RATE	9600

#define SERIAL_BUFSIZE_TX	256
#define SERIAL_BUFSIZE_RX	256

/* Structs */

struct serial_port {
	volatile unsigned char tx_buf[SERIAL_BUFSIZE_TX];
	volatile unsigned char rx_buf[SERIAL_BUFSIZE_RX];
	volatile unsigned int rx_head, rx_tail;
	volatile unsigned int tx_head, tx_tail, tx_tail_dma;
	unsigned int rx_ofs;
};

/* Externs */

extern struct serial_port p;
extern volatile int serial_tx_entry;

unsigned char serial_read(void);
void serial_putc(const char);
void serial_write(const char *, uint32_t);
void serial_print(const char *);
void __attribute__((format(printf,1,2))) serial_printf(const char *, ...);
void serial_flush(void);
void serial_auto_baud(void);
void serial_init_core(uint32_t);
void serial_init(void);
void hex_dump(const void *, size_t);

/* Macros */

static inline int serial_rx_ready() {
	return (SERIAL_DMA_RX->NDTR & DMA_SxNDT) != p.rx_ofs;
}

static inline int serial_tx_empty() {
	return SERIAL_UART->SR & USART_SR_TC;
}

#endif
